"""Constants used by pyTibber"""
from http import HTTPStatus
from typing import Final

__version__ = "0.27.0"

RESOLUTION_HOURLY: Final = "HOURLY"
RESOLUTION_DAILY: Final = "DAILY"
RESOLUTION_WEEKLY: Final = "WEEKLY"
RESOLUTION_MONTHLY: Final = "MONTHLY"
RESOLUTION_ANNUAL: Final = "ANNUAL"

DEMO_TOKEN: Final = "5K4MVS-OjfWhK_4yrjOlFe1F6kJXPVf7eQYggo8ebAE"
API_ENDPOINT: Final = "https://api.tibber.com/v1-beta/gql"

API_ERR_CODE_UNKNOWN = "UNKNOWN"
API_ERR_CODE_UNAUTH = "UNAUTHENTICATED"
HTTP_CODES_RETRIABLE = [HTTPStatus.TOO_MANY_REQUESTS, HTTPStatus.PRECONDITION_REQUIRED]
HTTP_CODES_FATAL = [HTTPStatus.BAD_REQUEST]



"""Exceptions"""

from .const import API_ERR_CODE_UNKNOWN


class HttpException(Exception):
    """Exception base for HTTP errors

    :param status: http response code
    :param message: http response message if any
    :param extension_code: http response extension if any
    """

    def __init__(
        self,
        status: int,
        message: str = "HTTP error",
        extension_code: str = API_ERR_CODE_UNKNOWN,
    ):
        self.status = status
        self.message = message
        self.extension_code = extension_code
        super().__init__(self.message)


class FatalHttpException(HttpException):
    """Exception raised for HTTP codes that are non-retriable"""


class RetryableHttpException(HttpException):
    """Exception raised for HTTP codes that are possible to retry"""


class InvalidLogin(FatalHttpException):
    """Invalid login exception."""



"""Websocket transport for Tibber."""
import asyncio
import datetime as dt
import logging

from gql.transport.exceptions import TransportClosed
from gql.transport.websockets import WebsocketsTransport

_LOGGER = logging.getLogger(__name__)


class TibberWebsocketsTransport(WebsocketsTransport):
    """Tibber websockets transport."""

    def __init__(self, url: str, access_token: str, user_agent: str) -> None:
        """Initialize TibberWebsocketsTransport."""
        super().__init__(
            url=url,
            init_payload={"token": access_token},
            headers={"User-Agent": user_agent},
            ping_interval=10,
        )
        self._timeout: int = 90
        self.reconnect_at: dt.datetime = dt.datetime.now() + dt.timedelta(
            seconds=self._timeout
        )

    @property
    def running(self) -> bool:
        """Is real time subscription running."""
        return (
            self.websocket is not None
            and self.websocket.open
            and self.reconnect_at > dt.datetime.now()
            and self.receive_data_task in asyncio.all_tasks()
        )

    async def _receive(self) -> str:
        """Wait the next message from the websocket connection."""
        try:
            msg = await asyncio.wait_for(super()._receive(), timeout=self._timeout)
        except asyncio.TimeoutError:
            _LOGGER.error("No data received from Tibber for %s seconds", self._timeout)
            raise
        self.reconnect_at = dt.datetime.now() + dt.timedelta(seconds=self._timeout)
        return msg

    async def close(self) -> None:
        await self._fail(TransportClosed("Tibber websocket closed by pyTibber"))
        await self.wait_closed()
